(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('lodash'), require('dayjs')) :
  typeof define === 'function' && define.amd ? define(['exports', 'lodash', 'dayjs'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.ZhuxUtils = {}, global.lodash, global.dayjs));
})(this, (function (exports, lodash, dayjs) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var dayjs__default = /*#__PURE__*/_interopDefaultLegacy(dayjs);

  const hashCacheKey = (cacheKey) => {
    return JSON.stringify(
      cacheKey,
      (_, val) => lodash.isPlainObject(val) ? Object.keys(val).sort().reduce((result, key) => {
        result[key] = val[key];
        return result;
      }, {}) : val
    );
  };
  class CacheHelper {
    constructor() {
      this.store = {};
      this.add = (key, value) => {
        const hashKey = hashCacheKey(key);
        this.store[hashKey] = value;
        return value;
      };
      this.remove = (key) => {
        const hashKey = hashCacheKey(key);
        Reflect.deleteProperty(this.store, hashKey);
      };
      this.clear = () => this.store = {};
    }
    async get(key, cb) {
      const hashKey = hashCacheKey(key);
      if (!this.store[hashKey])
        this.store[hashKey] = await (cb == null ? void 0 : cb());
      return this.store[hashKey];
    }
  }

  var __defProp$1 = Object.defineProperty;
  var __defProps$1 = Object.defineProperties;
  var __getOwnPropDescs$1 = Object.getOwnPropertyDescriptors;
  var __getOwnPropSymbols$1 = Object.getOwnPropertySymbols;
  var __hasOwnProp$1 = Object.prototype.hasOwnProperty;
  var __propIsEnum$1 = Object.prototype.propertyIsEnumerable;
  var __defNormalProp$1 = (obj, key, value) => key in obj ? __defProp$1(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
  var __spreadValues$1 = (a, b) => {
    for (var prop in b || (b = {}))
      if (__hasOwnProp$1.call(b, prop))
        __defNormalProp$1(a, prop, b[prop]);
    if (__getOwnPropSymbols$1)
      for (var prop of __getOwnPropSymbols$1(b)) {
        if (__propIsEnum$1.call(b, prop))
          __defNormalProp$1(a, prop, b[prop]);
      }
    return a;
  };
  var __spreadProps$1 = (a, b) => __defProps$1(a, __getOwnPropDescs$1(b));
  const bfsTree = (tree, cb, options = {}) => {
    const { childrenStr = "children" } = options;
    const queue = [...tree];
    while (queue.length > 0) {
      const node = queue.shift();
      if (cb(node) === false)
        break;
      Array.isArray(node[childrenStr]) && queue.push(...node[childrenStr]);
    }
  };
  const dfsTree = (tree, cb, options = {}) => {
    const { childrenStr = "children" } = options;
    const stack = [...tree];
    while (stack.length > 0) {
      const node = stack.pop();
      if (cb(node) === false)
        break;
      if (Array.isArray(node[childrenStr]))
        stack.push(...node[childrenStr].reverse());
    }
  };
  const TreeTransform = {
    pid2children(data, options = {}) {
      const { idStr = "id", pidStr = "pid", childrenStr = "children" } = options;
      const map = {}, res = [], _data = lodash.cloneDeep(data);
      _data.forEach((item) => map[item[idStr]] = item);
      _data.forEach((item) => {
        const p = map[item[pidStr]];
        if (p)
          p[childrenStr] ? p[childrenStr].push(item) : p[childrenStr] = [item];
        else
          res.push(item);
      });
      return res;
    },
    children2pid(data, options = {}) {
      const resArr = [];
      const _flatTreeArr = (data2, options2) => {
        const { idStr = "id", pidStr = "pid", childrenStr = "children", pid } = options2;
        data2.forEach((item) => {
          const tempItem = lodash.cloneDeep(item);
          if (!lodash.isNil(pid))
            tempItem[pidStr] = pid;
          Reflect.deleteProperty(tempItem, childrenStr);
          resArr.push(tempItem);
          if (Array.isArray(item[childrenStr]) && item[childrenStr].length > 0) {
            _flatTreeArr(item[childrenStr], __spreadProps$1(__spreadValues$1({}, options2), { pid: item[idStr] }));
          }
        });
      };
      _flatTreeArr(data, options);
      return resArr;
    }
  };
  var TreeUtil = {
    bfsTree,
    dfsTree,
    TreeTransform
  };

  var __defProp = Object.defineProperty;
  var __defProps = Object.defineProperties;
  var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
  var __getOwnPropSymbols = Object.getOwnPropertySymbols;
  var __hasOwnProp = Object.prototype.hasOwnProperty;
  var __propIsEnum = Object.prototype.propertyIsEnumerable;
  var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
  var __spreadValues = (a, b) => {
    for (var prop in b || (b = {}))
      if (__hasOwnProp.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    if (__getOwnPropSymbols)
      for (var prop of __getOwnPropSymbols(b)) {
        if (__propIsEnum.call(b, prop))
          __defNormalProp(a, prop, b[prop]);
      }
    return a;
  };
  var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
  const addCacheWrapper = (func) => {
    const cache = {};
    return function(...rest) {
      const key = JSON.stringify(rest);
      if (cache[key])
        return cache[key];
      cache[key] = func(...rest);
      return cache[key];
    };
  };
  const genSetRefObjFunc = (obj) => {
    return (key, value) => {
      if (typeof key === "string") {
        if (typeof value === "undefined")
          Reflect.deleteProperty(obj.current, key);
        else
          obj.current = __spreadProps(__spreadValues({}, obj.current), { [key]: value });
      } else if (typeof key === "function")
        obj.current = key(obj.current);
      else
        obj.current = __spreadValues(__spreadValues({}, obj.current), key);
    };
  };
  const genSetObjFunc = (obj) => {
    return (key, value) => {
      if (typeof key === "string") {
        if (typeof value === "undefined")
          Reflect.deleteProperty(obj, key);
        else
          obj[key] = value;
      } else if (typeof key === "function")
        key(obj);
      else
        Object.entries(key).map(([k, v]) => obj[k] = v);
    };
  };
  const parseJson = (json) => {
    let res = null;
    if (!json)
      return res;
    try {
      res = JSON.parse(json);
    } catch (e) {
      console.error(e);
    }
    return res;
  };
  const getDom = (id, className) => {
    let dom = document.querySelector(`#${id}`);
    if (!dom) {
      dom = document.createElement("div");
      dom.id = id;
      document.body.appendChild(dom);
    }
    className && dom.classList.add(className);
    return dom;
  };
  const abortablePromise = (promise) => {
    const abortController = new AbortController();
    const { signal } = abortController;
    return {
      promise: new Promise((resolve, reject) => {
        signal.addEventListener("abort", () => reject(signal == null ? void 0 : signal.reason));
        promise.then(resolve).catch(reject);
      }),
      abortController
    };
  };
  const clearEmptyVal = (obj, option = {}) => {
    const { clearEmptyStr = true, clearSpace = true } = option;
    if (!lodash.isPlainObject(obj))
      return obj;
    const newObj = lodash.cloneDeep(obj);
    const condition = (v, clearEmptyStr2 = true) => {
      let res = typeof v === "undefined" || v === null || v === "$$";
      clearEmptyStr2 && (res = res || v === "");
      return res;
    };
    Object.entries(newObj).forEach(([k, v]) => {
      if (clearSpace && typeof v === "string")
        newObj[k] = v.trim();
      if (condition(v, clearEmptyStr))
        Reflect.deleteProperty(newObj, k);
    });
    return newObj;
  };
  const toOptions = {
    arr2options(arr) {
      return arr.map((item) => ({ label: item, value: item }));
    },
    enum2options(e) {
      const keys = Object.keys(e).filter((item) => Number.isNaN(+item));
      return keys.map((item) => ({ value: e[item], label: item }));
    }
  };
  const subSet = (arr1, arr2) => {
    const set1 = new Set(arr1);
    const set2 = new Set(arr2);
    const result = [];
    set1.forEach((item) => {
      if (!set2.has(item))
        result.push(item);
    });
    return result;
  };
  const union = (arr1, arr2) => {
    const set1 = new Set(arr1);
    const set2 = new Set(arr2);
    const result = [];
    set1.forEach((item) => {
      if (set2.has(item))
        result.push(item);
    });
    return result;
  };
  const genMap = (map) => {
    return new Proxy(map, {
      get(target, prop) {
        var _a;
        if (!prop)
          return;
        return (_a = target[prop]) != null ? _a : prop;
      }
    });
  };
  const removeStr = (str, config) => {
    const { removeStart, removeEnd } = config;
    if (!removeStart && !removeEnd)
      return str;
    const regStrList = [];
    removeStart && regStrList.push(`^(${removeStart})+`);
    removeEnd && regStrList.push(`(${removeEnd})+$`);
    const reg = new RegExp(regStrList.join("|"), "g");
    return str.replace(reg, "");
  };
  var CommonUtil = __spreadValues({
    addCacheWrapper,
    abortablePromise,
    genSetRefObjFunc,
    genSetObjFunc,
    parseJson,
    getDom,
    clearEmptyVal,
    toOptions,
    subSet,
    union,
    genMap,
    removeStr
  }, TreeUtil);

  const getBase64 = (file) => {
    return new Promise((resolve, reject) => {
      const reader = new FileReader();
      reader.readAsDataURL(file);
      reader.onload = () => resolve(reader.result);
      reader.onerror = (error) => reject(error);
    });
  };
  const downloadFile = (blob, fileName) => {
    const link = document.createElement("a");
    const href = URL.createObjectURL(blob);
    link.download = fileName;
    link.style.display = "none";
    link.href = href;
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
    URL.revokeObjectURL(href);
  };
  const translateUrl = (fileName, type, baseUrl) => {
    if (!fileName)
      return fileName;
    const regExp = /(^http(s?):\/\/)|(^\/\/)/;
    if (type === "addPrefix")
      return regExp.test(fileName) ? fileName : baseUrl + fileName;
    return regExp.test(fileName) ? fileName.replace(baseUrl, "") : fileName;
  };
  const _loadImage = (src) => {
    const img = new Image();
    img.crossOrigin = "Anonymous";
    img.src = src;
    return new Promise((resolve, reject) => {
      img.onload = () => resolve(img);
      img.onerror = (err) => reject(err);
    });
  };
  var FileUtil = { getBase64, downloadFile, translateUrl, loadImage: CommonUtil.addCacheWrapper(_loadImage) };

  const genFormatMap = (formatList) => {
    const res = {};
    formatList.forEach((item) => {
      item.fields.forEach((field) => {
        res[field] = item.format;
      });
    });
    return res;
  };
  const formatData = ({ data, module, type, formatMap }) => {
    const rawData = lodash.cloneDeep(data);
    const result = {};
    Object.entries(rawData).forEach(([k, v]) => {
      const key = `${module}/${k}`;
      const format = formatMap[key];
      if (format == null ? void 0 : format[type])
        result[k] = format[type](v, result, rawData);
      else
        result[k] = v;
    });
    return result;
  };
  var FormatUtil = { genFormatMap, formatData };

  class ListenValueHelper {
    constructor() {
      this.valueMap = new Proxy(
        {},
        {
          set: (obj, key, value) => {
            obj[key] = value;
            if (!lodash.isNil(value))
              this.doResolveFunc(key);
            return true;
          }
        }
      );
      this.resolveRejectMap = {};
      this.doResolveFunc = (key) => {
        const resolveRejectObj = this.resolveRejectMap[key];
        if (!resolveRejectObj)
          return;
        resolveRejectObj.resolveFuncList.forEach((item) => item == null ? void 0 : item(this.getValue(key)));
        resolveRejectObj.rejectTimerList.forEach((item) => item && clearTimeout(item));
        resolveRejectObj.resolveFuncList = [];
        resolveRejectObj.rejectTimerList = [];
      };
      this.addListener = (key, timeout, signal) => {
        if (!this.resolveRejectMap[key])
          this.resolveRejectMap[key] = { resolveFuncList: [], rejectTimerList: [] };
        return new Promise((resolve, reject) => {
          signal == null ? void 0 : signal.throwIfAborted();
          signal == null ? void 0 : signal.addEventListener("abort", () => reject(signal == null ? void 0 : signal.reason));
          if (!lodash.isNil(this.valueMap[key]))
            resolve(this.valueMap[key]);
          else {
            const { resolveFuncList, rejectTimerList } = this.resolveRejectMap[key];
            const resolveFunc = (value) => resolve(value);
            resolveFuncList.push(resolveFunc);
            if (timeout && timeout > 0) {
              const timer = setTimeout(() => {
                reject(new Error("timeout"));
                const index = resolveFuncList.findIndex((item) => item === resolveFunc);
                if (~index)
                  resolveFuncList[index] = null;
              }, timeout);
              rejectTimerList.push(timer);
            }
          }
        });
      };
      this.getValue = (key) => this.valueMap[key];
      this.setValue = (key, value) => this.valueMap[key] = value;
    }
  }

  var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

  var duration$1 = {exports: {}};

  (function (module, exports) {
  	!function(t,s){module.exports=s();}(commonjsGlobal,(function(){var t,s,n=1e3,i=6e4,e=36e5,r=864e5,o=/\[([^\]]+)]|Y{1,4}|M{1,4}|D{1,2}|d{1,4}|H{1,2}|h{1,2}|a|A|m{1,2}|s{1,2}|Z{1,2}|SSS/g,u=31536e6,h=2592e6,a=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/,d={years:u,months:h,days:r,hours:e,minutes:i,seconds:n,milliseconds:1,weeks:6048e5},c=function(t){return t instanceof p},f=function(t,s,n){return new p(t,n,s.$l)},m=function(t){return s.p(t)+"s"},l=function(t){return t<0},$=function(t){return l(t)?Math.ceil(t):Math.floor(t)},y=function(t){return Math.abs(t)},g=function(t,s){return t?l(t)?{negative:!0,format:""+y(t)+s}:{negative:!1,format:""+t+s}:{negative:!1,format:""}},p=function(){function l(t,s,n){var i=this;if(this.$d={},this.$l=n,void 0===t&&(this.$ms=0,this.parseFromMilliseconds()),s)return f(t*d[m(s)],this);if("number"==typeof t)return this.$ms=t,this.parseFromMilliseconds(),this;if("object"==typeof t)return Object.keys(t).forEach((function(s){i.$d[m(s)]=t[s];})),this.calMilliseconds(),this;if("string"==typeof t){var e=t.match(a);if(e){var r=e.slice(2).map((function(t){return null!=t?Number(t):0}));return this.$d.years=r[0],this.$d.months=r[1],this.$d.weeks=r[2],this.$d.days=r[3],this.$d.hours=r[4],this.$d.minutes=r[5],this.$d.seconds=r[6],this.calMilliseconds(),this}}return this}var y=l.prototype;return y.calMilliseconds=function(){var t=this;this.$ms=Object.keys(this.$d).reduce((function(s,n){return s+(t.$d[n]||0)*d[n]}),0);},y.parseFromMilliseconds=function(){var t=this.$ms;this.$d.years=$(t/u),t%=u,this.$d.months=$(t/h),t%=h,this.$d.days=$(t/r),t%=r,this.$d.hours=$(t/e),t%=e,this.$d.minutes=$(t/i),t%=i,this.$d.seconds=$(t/n),t%=n,this.$d.milliseconds=t;},y.toISOString=function(){var t=g(this.$d.years,"Y"),s=g(this.$d.months,"M"),n=+this.$d.days||0;this.$d.weeks&&(n+=7*this.$d.weeks);var i=g(n,"D"),e=g(this.$d.hours,"H"),r=g(this.$d.minutes,"M"),o=this.$d.seconds||0;this.$d.milliseconds&&(o+=this.$d.milliseconds/1e3);var u=g(o,"S"),h=t.negative||s.negative||i.negative||e.negative||r.negative||u.negative,a=e.format||r.format||u.format?"T":"",d=(h?"-":"")+"P"+t.format+s.format+i.format+a+e.format+r.format+u.format;return "P"===d||"-P"===d?"P0D":d},y.toJSON=function(){return this.toISOString()},y.format=function(t){var n=t||"YYYY-MM-DDTHH:mm:ss",i={Y:this.$d.years,YY:s.s(this.$d.years,2,"0"),YYYY:s.s(this.$d.years,4,"0"),M:this.$d.months,MM:s.s(this.$d.months,2,"0"),D:this.$d.days,DD:s.s(this.$d.days,2,"0"),H:this.$d.hours,HH:s.s(this.$d.hours,2,"0"),m:this.$d.minutes,mm:s.s(this.$d.minutes,2,"0"),s:this.$d.seconds,ss:s.s(this.$d.seconds,2,"0"),SSS:s.s(this.$d.milliseconds,3,"0")};return n.replace(o,(function(t,s){return s||String(i[t])}))},y.as=function(t){return this.$ms/d[m(t)]},y.get=function(t){var s=this.$ms,n=m(t);return "milliseconds"===n?s%=1e3:s="weeks"===n?$(s/d[n]):this.$d[n],0===s?0:s},y.add=function(t,s,n){var i;return i=s?t*d[m(s)]:c(t)?t.$ms:f(t,this).$ms,f(this.$ms+i*(n?-1:1),this)},y.subtract=function(t,s){return this.add(t,s,!0)},y.locale=function(t){var s=this.clone();return s.$l=t,s},y.clone=function(){return f(this.$ms,this)},y.humanize=function(s){return t().add(this.$ms,"ms").locale(this.$l).fromNow(!s)},y.milliseconds=function(){return this.get("milliseconds")},y.asMilliseconds=function(){return this.as("milliseconds")},y.seconds=function(){return this.get("seconds")},y.asSeconds=function(){return this.as("seconds")},y.minutes=function(){return this.get("minutes")},y.asMinutes=function(){return this.as("minutes")},y.hours=function(){return this.get("hours")},y.asHours=function(){return this.as("hours")},y.days=function(){return this.get("days")},y.asDays=function(){return this.as("days")},y.weeks=function(){return this.get("weeks")},y.asWeeks=function(){return this.as("weeks")},y.months=function(){return this.get("months")},y.asMonths=function(){return this.as("months")},y.years=function(){return this.get("years")},y.asYears=function(){return this.as("years")},l}();return function(n,i,e){t=e,s=e().$utils(),e.duration=function(t,s){var n=e.locale();return f(t,{$l:n},s)},e.isDuration=c;var r=i.prototype.add,o=i.prototype.subtract;i.prototype.add=function(t,s){return c(t)&&(t=t.asMilliseconds()),r.bind(this)(t,s)},i.prototype.subtract=function(t,s){return c(t)&&(t=t.asMilliseconds()),o.bind(this)(t,s)};}}));
  } (duration$1));

  var duration = duration$1.exports;

  dayjs__default["default"].extend(duration);
  class StorageHelper {
    constructor(prefix, type = "local") {
      this._prefix = prefix ? `${prefix}_` : prefix;
      this._storage = type === "local" ? globalThis.localStorage : globalThis.sessionStorage;
    }
    get length() {
      return this._storage.length;
    }
    clear() {
      this._storage.clear();
    }
    key(index) {
      return this._storage.key(index);
    }
    removeItem(key) {
      key = this._prefix + key;
      this._storage.removeItem(key);
    }
    getItem(key) {
      const data = this._storage.getItem(this._prefix + key);
      if (!data)
        return data;
      const dataObj = CommonUtil.parseJson(data);
      if (!dataObj)
        return data;
      if (!dataObj._expire || typeof dataObj._expire !== "number" || !("value" in dataObj))
        return data;
      if (dataObj._expire < Date.now()) {
        this.removeItem(key);
        return null;
      }
      return dataObj.value;
    }
    setItem(key, value, _expire) {
      if (lodash.isNil(value))
        return;
      key = this._prefix + key;
      const data = { value, _expire };
      if (_expire) {
        const ms = typeof _expire === "number" ? _expire : dayjs__default["default"].duration(_expire).asMilliseconds();
        data._expire = Date.now() + ms;
      } else
        data._expire = Number.MAX_SAFE_INTEGER;
      this._storage.setItem(key, JSON.stringify(data));
    }
  }

  exports.CacheHelper = CacheHelper;
  exports.CommonUtil = CommonUtil;
  exports.FileUtil = FileUtil;
  exports.FormatUtil = FormatUtil;
  exports.ListenValueHelper = ListenValueHelper;
  exports.StorageHelper = StorageHelper;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
