import { isPlainObject } from "lodash";
const hashCacheKey = (cacheKey) => {
    return JSON.stringify(cacheKey, (_, val) => isPlainObject(val)
        ? Object.keys(val)
            .sort()
            .reduce((result, key) => {
            result[key] = val[key];
            return result;
        }, {})
        : val);
};
class CacheHelper {
    constructor() {
        this.store = {};
        this.add = (key, value) => {
            const hashKey = hashCacheKey(key);
            this.store[hashKey] = value;
            return value;
        };
        this.remove = (key) => {
            const hashKey = hashCacheKey(key);
            Reflect.deleteProperty(this.store, hashKey);
        };
        this.clear = () => (this.store = {});
    }
    async get(key, cb) {
        const hashKey = hashCacheKey(key);
        if (!this.store[hashKey])
            this.store[hashKey] = await (cb === null || cb === void 0 ? void 0 : cb());
        return this.store[hashKey];
    }
}
export default CacheHelper;
