import { cloneDeep, isPlainObject } from "lodash";
import TreeUtil from "./TreeUtil";
const addCacheWrapper = (func) => {
    const cache = {};
    return function (...rest) {
        const key = JSON.stringify(rest);
        if (cache[key])
            return cache[key];
        cache[key] = func(...rest);
        return cache[key];
    };
};
const genSetRefObjFunc = (obj) => {
    return (key, value) => {
        if (typeof key === "string") {
            if (typeof value === "undefined")
                Reflect.deleteProperty(obj.current, key);
            else
                obj.current = Object.assign(Object.assign({}, obj.current), { [key]: value });
        }
        else if (typeof key === "function")
            obj.current = key(obj.current);
        else
            obj.current = Object.assign(Object.assign({}, obj.current), key);
    };
};
const genSetObjFunc = (obj) => {
    return (key, value) => {
        if (typeof key === "string") {
            if (typeof value === "undefined")
                Reflect.deleteProperty(obj, key);
            else
                obj[key] = value;
        }
        else if (typeof key === "function")
            key(obj);
        else
            Object.entries(key).map(([k, v]) => (obj[k] = v));
    };
};
const parseJson = (json) => {
    let res = null;
    if (!json)
        return res;
    try {
        res = JSON.parse(json);
    }
    catch (e) {
        console.error(e);
    }
    return res;
};
const getDom = (id, className) => {
    let dom = document.querySelector(`#${id}`);
    if (!dom) {
        dom = document.createElement("div");
        dom.id = id;
        document.body.appendChild(dom);
    }
    className && dom.classList.add(className);
    return dom;
};
// https://juejin.cn/post/7140558929750130719
const abortablePromise = (promise) => {
    const abortController = new AbortController();
    const { signal } = abortController;
    return {
        promise: new Promise((resolve, reject) => {
            signal.addEventListener("abort", () => reject(signal === null || signal === void 0 ? void 0 : signal.reason));
            promise.then(resolve).catch(reject);
        }),
        abortController,
    };
};
const clearEmptyVal = (obj, option = {}) => {
    const { clearEmptyStr = true, clearSpace = true } = option;
    if (!isPlainObject(obj))
        return obj;
    const newObj = cloneDeep(obj);
    const condition = (v, clearEmptyStr = true) => {
        let res = typeof v === "undefined" || v === null || v === "$$";
        clearEmptyStr && (res = res || v === "");
        return res;
    };
    Object.entries(newObj).forEach(([k, v]) => {
        if (clearSpace && typeof v === "string")
            newObj[k] = v.trim();
        if (condition(v, clearEmptyStr))
            Reflect.deleteProperty(newObj, k);
    });
    return newObj;
};
const toOptions = {
    arr2options(arr) {
        return arr.map(item => ({ label: item, value: item }));
    },
    enum2options(e) {
        const keys = Object.keys(e).filter(item => Number.isNaN(+item));
        return keys.map(item => ({ value: e[item], label: item }));
    },
};
/** 求差集 第一个减去第二个 */
const subSet = (arr1, arr2) => {
    const set1 = new Set(arr1);
    const set2 = new Set(arr2);
    const result = [];
    set1.forEach(item => {
        if (!set2.has(item))
            result.push(item);
    });
    return result;
};
const union = (arr1, arr2) => {
    const set1 = new Set(arr1);
    const set2 = new Set(arr2);
    const result = [];
    set1.forEach(item => {
        if (set2.has(item))
            result.push(item);
    });
    return result;
};
const genMap = (map) => {
    return new Proxy(map, {
        get(target, prop) {
            var _a;
            if (!prop)
                return;
            return (_a = target[prop]) !== null && _a !== void 0 ? _a : prop;
        },
    });
};
const removeStr = (str, config) => {
    const { removeStart, removeEnd } = config;
    if (!removeStart && !removeEnd)
        return str;
    const regStrList = [];
    removeStart && regStrList.push(`^(${removeStart})+`);
    removeEnd && regStrList.push(`(${removeEnd})+$`);
    const reg = new RegExp(regStrList.join("|"), "g");
    return str.replace(reg, "");
};
export default Object.assign({ addCacheWrapper,
    abortablePromise,
    genSetRefObjFunc,
    genSetObjFunc,
    parseJson,
    getDom,
    clearEmptyVal,
    toOptions,
    subSet,
    union,
    genMap,
    removeStr }, TreeUtil);
