import { cloneDeep, isNil } from "lodash";
const bfsTree = (tree, cb, options = {}) => {
    const { childrenStr = "children" } = options;
    const queue = [...tree];
    while (queue.length > 0) {
        const node = queue.shift();
        if (cb(node) === false)
            break;
        Array.isArray(node[childrenStr]) && queue.push(...node[childrenStr]);
    }
};
const dfsTree1 = (tree, cb, options = {}) => {
    const { childrenStr = "children" } = options;
    const _dfs = (tree, cb) => {
        for (const node of tree) {
            cb(node);
            if (Array.isArray(node[childrenStr]))
                _dfs(node[childrenStr], cb);
        }
    };
    _dfs(tree, cb);
};
const dfsTree = (tree, cb, options = {}) => {
    const { childrenStr = "children" } = options;
    const stack = [...tree];
    while (stack.length > 0) {
        const node = stack.pop();
        if (cb(node) === false)
            break;
        if (Array.isArray(node[childrenStr]))
            stack.push(...node[childrenStr].reverse());
    }
};
const TreeTransform = {
    pid2children(data, options = {}) {
        const { idStr = "id", pidStr = "pid", childrenStr = "children" } = options;
        const map = {}, res = [], _data = cloneDeep(data);
        _data.forEach(item => (map[item[idStr]] = item));
        _data.forEach(item => {
            const p = map[item[pidStr]];
            if (p)
                p[childrenStr] ? p[childrenStr].push(item) : (p[childrenStr] = [item]);
            else
                res.push(item);
        });
        return res;
    },
    children2pid(data, options = {}) {
        const resArr = [];
        const _flatTreeArr = (data, options) => {
            const { idStr = "id", pidStr = "pid", childrenStr = "children", pid } = options;
            data.forEach(item => {
                const tempItem = cloneDeep(item);
                if (!isNil(pid))
                    tempItem[pidStr] = pid;
                Reflect.deleteProperty(tempItem, childrenStr);
                resArr.push(tempItem);
                if (Array.isArray(item[childrenStr]) && item[childrenStr].length > 0) {
                    _flatTreeArr(item[childrenStr], Object.assign(Object.assign({}, options), { pid: item[idStr] }));
                }
            });
        };
        _flatTreeArr(data, options);
        return resArr;
    },
};
export default {
    bfsTree,
    dfsTree,
    TreeTransform,
};
